; ***************************************************************************************
; ***                                                                                 ***
; *** This code requires a modified DASM which was posted to [stella] at 02/19/2001 ! ***
; ***                                                                                 ***
; ***************************************************************************************

; *** DEATH DERBY ***

; 01/19/2003:
; - first working 2LK demo version with preliminary grenade
; 01/19/2003 (later):
; - improved grenade code:
;   - no safe zone colouring
;   - taler ball
;   - fullscreen ball mode switchable with right difficulty
; 01/26/2003:
; - optimized kernel, *no* tearing of zombies and cars
; - kernel initialisation code added, all four objects can be freely moved
;   *everywhere* (DEBUG: use left, right joystick and left fire button)
;   TODO: correct X-position if zombie enters at top of screen
; - reduced zombie RAM usage
; - halved tombstone RAM usge


    processor 6502
    include vcs.h

;===============================================================================
; T I A - C O N S T A N T S
;===============================================================================

BM_SIZE_1   = %000000
BM_SIZE_2   = %010000
BM_SIZE_4   = %100000
BM_SIZE_8   = %110000

HMOVE_L7    = $70
HMOVE_L6    = $60
HMOVE_L5    = $50
HMOVE_L4    = $40
HMOVE_L3    = $30
HMOVE_L2    = $20
HMOVE_L1    = $10
HMOVE_0     = $00
HMOVE_R1    = $f0
HMOVE_R2    = $e0
HMOVE_R3    = $d0
HMOVE_R4    = $c0
HMOVE_R5    = $b0
HMOVE_R6    = $a0
HMOVE_R7    = $90
HMOVE_R8    = $80

DISABLE_BM  = %00
ENABLE_BM   = %10

;===============================================================================
; U S E R - C O N S T A N T S
;===============================================================================

SCREEN_W        = 160

TOMBSTONEHEIGHT = 8  ; 8*2 = 16 (single scanlines) high per tombstone
TOMBSTONEROWS   = 11 ; 11 rows of tombstones per screen

CARHEIGHT       = 8  ; 8 doubled scanlines (used as offset)
ZOMBIE_H        = 8

KERNELHEIGHT    = TOMBSTONEROWS*TOMBSTONEHEIGHT+CARHEIGHT

CAR0_COL        = $36;
CAR1_COL        = $c6;

GRENADE_COL     = $58


RAND_EOR_8      = $b2

;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

frameCnt    ds 1
tmpVar      ds 1
random      ds 1

tmpPF_L     ds 1
tmpPF_R     ds 1

dataPF_L    ds TOMBSTONEROWS
dataPF_R    ds TOMBSTONEROWS

P0_Y        ds 1
P1_Y        ds 1
M0_Y        ds 1
M1_Y        ds 1
M0_YTmp     ds 1
M1_YTmp     ds 1

P0_X        ds 1
P1_X        ds 1
M0_X        ds 1
M1_X        ds 1
BL_X        ds 1

P0_YFullres ds 1                ; 0..KERNELHEIGHT*2-2
P1_YFullres ds 1

P0_Ptr      ds 2
P1_Ptr      ds 2
M0_Ptr      ds 2
M1_Ptr      ds 2

car0Id      ds 1
car1Id      ds 1
zombie0Id   ds 1
zombie1Id   ds 1

grenadeRow  ds 1
grenadeClr  ds 1        ; bit 0 = 1 -> full screen



;===============================================================================
; M A C R O S
;===============================================================================

  MAC BIT_B
    .byte   $24
  ENDM

  MAC BIT_W
    .byte   $2c
  ENDM

  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================

    SEG     Bank0
    ORG     $f000

Start SUBROUTINE
    sei                         ;           Disable interrupts, if there are any.
    cld                         ;           Clear BCD math bit.
    ldx     #0
    txs
    pha                         ;           Set stack to beginning.
    txa
.clearLoop:
    pha
    dex
    bne     .clearLoop

    jsr     GameInit

.mainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    jsr     OverScan
    jmp     .mainLoop


GameInit SUBROUTINE
    lda     #1
    sta     random

    ldx     #TOMBSTONEROWS*2-1
    lda     #$0
.loopFill:
    sta     dataPF_L,x
    dex
    bpl     .loopFill

    lda     #BM_SIZE_2|1
    sta     CTRLPF

    lda     #$00
    sta     COLUBK
    lda     #CAR0_COL   ;$36
    sta     COLUP0
    lda     #CAR1_COL   ;$c6
    sta     COLUP1

    lda     #20          ;KERNELHEIGHT*2-2;16
    sta     P0_YFullres
    lda     #30
    sta     P1_YFullres

    lda     #116
    sta     P0_X
    lda     #132
    sta     P1_X

    lda     #70|$80
    sta     M0_Y
    lda     #20|$80
    sta     M1_Y

    lda     #80
    sta     M0_X
    lda     #88
    sta     M1_X

    lda     #50
    sta     BL_X

    lda     #>CarRotation0
    sta     P0_Ptr+1
    sta     P1_Ptr+1

    lda     #>Ped1_1
    sta     M0_Ptr+1
    sta     M1_Ptr+1

    ldx     #0
    stx     car0Id
    ldx     #8
    stx     car1Id

    ldx     #0
    stx     zombie0Id
    ldx     #3
    stx     zombie1Id

    lda     #6
    sta     grenadeRow

    lda     #1
    sta     VDELBL

    rts


VerticalBlank SUBROUTINE
    lda     #2
    sta     WSYNC
    sta     VSYNC
    sta     WSYNC
    sta     WSYNC
    lsr
    ldx     #44
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
    rts


GameCalc SUBROUTINE
    inc     frameCnt

; randomly fill playfield:
    lda     frameCnt
    and     #$1f
    bne     .skipRandom
.retryRandom:
    jsr     NextRandom
    lda     random
    tax
    and     #7
    tay
    txa
    lsr
    lsr
    lsr
    and     #$1f
    cmp     #TOMBSTONEROWS*2
    bcs     .retryRandom
    tax
    lda     dataPF_L,x
    ora     Pot2Tbl,y
    sta     dataPF_L,x
    jsr     NextRandom
.skipRandom:

; move the ball:
    lda     frameCnt
    and     #$03
    bne     .skipMoveBL

    ldx     BL_X
    dex
    cpx     #17
    bcs     .okBL_X
    ldx     #143
.okBL_X:
    stx     BL_X
.skipMoveBL:


; *** calculate kernel variables: ***
; calculate pointer to car #0 graphics:
    lda     P0_YFullres
    sta     VDELP0
    lsr
    sta     P0_Y

    ldy     car0Id
    lda     CarTbl,y
    clc
    adc     P0_Y
    sta     P0_Ptr

; calculate pointer to car #1 graphics:
    lda     P1_YFullres
    eor     #1
    sta     VDELP1
    eor     #1
    lsr
    adc     #0
    sta     P1_Y

    ldy     car1Id
    lda     CarTbl,y
    clc
    adc     P1_Y
    sta     P1_Ptr

; calculate pointer to missile #0 graphics:
    lda     M0_Y
    and     #~$80
    clc
    adc     #ZOMBIE_H-1
    sta     M0_YTmp
    tay

    ldx     zombie0Id
    lda     Ped1_Tbl,x
    sec
    sbc     M0_YTmp
    sta     M0_Ptr

    lda     (M0_Ptr),y
    asl
    asl
    sta     NUSIZ0

; calculate pointer to missile #1 graphics:
    lda     M1_Y
    and     #~$80
    clc
    adc     #ZOMBIE_H-1
    sta     M1_YTmp
    tay

    ldx     zombie1Id
    lda     Ped1_Tbl,x
    sec
    sbc     M1_YTmp
    sta     M1_Ptr

    lda     (M1_Ptr),y
    asl
    asl
    sta     NUSIZ1

; position all objects
    ldx     #4
.loopPos:
    lda     P0_X,x
    jsr     SetPosX
    dex
    bpl     .loopPos
    sta     WSYNC

    sta     HMOVE
    sta     WSYNC

    sta     HMCLR
    rts


NextRandom SUBROUTINE
;    lda     random              ; 3
;    lsr                         ; 2
;    bcc     .skipEor            ; 2
;    eor     #RAND_EOR_8         ; 2
;.skipEor:                       ;   = 11/12 (~11.5)
;    sta     random              ; 3

    lda    random               ; 3
    asl                         ; 2
    eor    random               ; 3
    asl                         ; 2
    eor    random               ; 3
    asl                         ; 2
    asl                         ; 2
    eor    random               ; 3
    asl                         ; 2
    rol    random               ; 5
    rts                         ; 6
;NextRandom END


    align 256

DrawScreen SUBROUTINE
; setting up a timer to make experimenting inside the kernel easier
; (the final code shouldm't need this):
    ldx     #228
.waitTim:
    lda     INTIM
    bne     .waitTim
    sta     WSYNC
    sta     VBLANK
    stx     TIM64T

; *** initialize the kernel loop: ***
;    lda     MaskTbl+TOMBSTONEROWS-1 ; 4
    lda     #%01010101
    and     dataPF_L+TOMBSTONEROWS-1; 4
    sta     tmpPF_L                 ; 3
    lda     dataPF_R+TOMBSTONEROWS-1; 4
    sta     tmpPF_R                 ; 3 = 18

    lda     #0              ; 2
    sta     COLUP0          ; 3
    sta     COLUP1          ; 3 =  8

    ldy     #KERNELHEIGHT-1 ; 2 =  4

    cpy     M0_YTmp         ; 3
    bpl     .skipEnableM0   ; 2
    lda     M0_Y            ; 3
    sta     M0_YTmp         ; 3
    lda     #2              ; 2
    sta     ENAM0           ; 3
.skipEnableM0:

    cpy     M1_YTmp         ; 3
    bpl     .skipEnableM1   ; 2
    lda     M1_Y            ; 3
    sta     M1_YTmp         ; 3
    lda     #2              ; 2
    sta     ENAM1           ; 3
.skipEnableM1:

    iny                     ; 2
    ldx     #TOMBSTONEROWS-1; 2 =  4

; draw car #1:
    lda     #CARHEIGHT-1    ; 2
    cmp     P1_Y            ; 3
    bcs     .doDraw1        ; 2
    lda     #0              ; 2
    BIT_W                   ;-1
.doDraw1:
    lda     (P1_Ptr),y      ; 5
    sta     GRP1            ; 3 = 16

    lda     #CARHEIGHT-1    ; 2
    cmp     P0_Y            ; 3
    bcs     .doDraw0        ; 2
    lda     #0              ; 2
    BIT_W                   ;-1
.doDraw0:
    lda     (P0_Ptr),y      ; 5
    sta     GRP0            ; 3 = 16

    dey                     ; 2 =  2

; draw car #1:
    lda     #CARHEIGHT-1    ; 2
    dcp     P1_Y            ; 5
    bcs     .doDraw1_a      ; 2
    lda     #0              ; 2
    BIT_W                   ;-1
.doDraw1_a:
    lda     (P1_Ptr),y      ; 5
    sta     GRP1            ; 3 = 18

    lda     grenadeClr      ; 3
    asl                     ; 2 =  5

    sta     WSYNC
;---------------------------------------
    sta     ENABL           ; 3 =  3            VDELed

    lda     #CAR1_COL       ; 2
    sta     COLUP1          ; 3
    lda     #CAR0_COL       ; 2
    sta     COLUP0          ; 3
    lda     #$90            ; 2                 enable left and right borders
    sta     PF0             ; 3
    jmp     EnterKernel     ; 3 = 18    @21
;***************************************

    align   256

; wasting some bytes to avoid page penalties:
    ds      $74, 0

Kernel SUBROUTINE

.disableM1:                 ; 9
    SLEEP   2               ; 2                 waste two cycles for constant timing
    lda     #DISABLE_BM     ; 2
    beq     .continueM1a    ;10

.checkM1:                   ; 6
    bne     .disableM1      ; 2
;.enableM1:
    lda     M1_Y            ; 3
    sta     M1_YTmp         ; 3
    lda     #ENABLE_BM      ; 2
.continueM1a:
    sta.w   ENAM1           ; 4         @00     (>= @00)
    bpl     .continueM1     ; 3

.disableM0:                 ; 8
    SLEEP   2               ; 5                 waste two cycles for constant timing
    lda     #DISABLE_BM     ; 2
    beq     .continueM0a    ; 9

.checkM0:                   ; 6
    bne     .disableM0      ; 2
;.enableM0:
    lda     M0_Y            ; 3
    sta     M0_YTmp         ; 3
    lda     #ENABLE_BM      ; 2
.continueM0a:
    sta.w   ENAM0           ; 4         @01     (>= @00)
    bpl     .continueM0     ; 3

;***************************************


; cycle count (tombstone row):
; - loop overhead            9
; - COLUPF             5 =   5
; - playfield       29+8 =  37
; - players         18*2 =  36
; - missiles/HMOVE  23*2 =  46
; - HMCLR            3*2 =   6
; - free             4+9 =  13
;                          ---
; Total             76*2 = 152 :-)

.contBlock:                 ;           @42
    SLEEP   9               ; 9
    asl                     ; 2 = 11

.nextBlock:
    sta     COLUPF          ; 3 =  3    @56     too early for 2nd row of ball

; draw zombie #1:
    cpy     M1_YTmp         ; 3
    bpl     .checkM1        ; 2
;.drawM1:
    lda     (M1_Ptr),y      ; 5
    sta     HMM1            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     HMOVE           ; 3         @00     three cycles too early, ok
;---------------------------------------
    sta     NUSIZ1          ; 3         @03     (>= @00)
.continueM1:                ;   = 23            timing is constant!

; draw car #1:
    lda     #CARHEIGHT-1    ; 2
    dcp     P1_Y            ; 5
    bcs     .doDraw1        ; 2
    lda     #0              ; 2
    BIT_W                   ;-1
.doDraw1:
    lda     (P1_Ptr),y      ; 5
    sta     GRP1            ; 3 = 18    @21
EnterKernel:              ;

; draw tomstones (part 1/2):
    lda     tmpPF_L         ; 3
    sta     PF1             ; 3         @27     (<= @27)
    eor     dataPF_L,x      ; 4
    sta     PF2             ; 3         @34     (<= @38)

    SLEEP   4               ; 4

; draw tomstones (part 2/2):
    lda     MaskTbl,x       ; 4
    and     tmpPF_R         ; 3
    sta     PF2             ; 3         @48     (== @48)
    eor     tmpPF_R         ; 3
    sta     PF1             ; 3         @54     (<= @54)

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @57

; draw zombie #0:
    cpy     M0_YTmp         ; 3
    bpl     .checkM0        ; 2
;.drawM0:
    lda     (M0_Ptr),y      ; 5
    sta     HMM0            ; 3
    asl                     ; 2
    asl                     ; 2
;---------------------------------------
    sta     HMOVE           ; 3         @01     two cycles too early, ok
    sta     NUSIZ0          ; 3         @04     (>= @00)
.continueM0:                ;   = 23            timing is constant!

; draw car #0:
    lda     #CARHEIGHT-1    ; 2
    dcp     P0_Y            ; 5
    bcs     .doDraw0        ; 2
    lda     #0              ; 2
    BIT_W                   ;-1
.doDraw0:
    lda     (P0_Ptr),y      ; 5
    sta     GRP0            ; 3 = 18    @22

; disable playfield:
    lda     #0              ; 2
    sta     PF1             ; 3         @27     (<= @27)
    sta     PF2             ; 3 =  8    @30     (<= @38)

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @33

; tombstone indexer routine (part 1/2):
    dey                     ; 2
    tya                     ; 2
    and     #7              ; 2
    bne     .contBlock      ; 2=  8

    SLEEP   5               ; 5 =  5

; draw grenade:
    lda     #ENABLE_BM      ; 2
    cpx     grenadeRow      ; 3
    beq     .enableBL       ; 2
    BIT_W                   ; 1
.enableBL:
    sta     ENABL           ; 3 = 11    @57     VDELed!

; draw zombie #1:
    cpy     M1_YTmp         ; 3
    bpl     .checkM1_a      ; 2
;.drawM1_a:
    lda     (M1_Ptr),y      ; 5
    sta     HMM1            ; 3
    asl                     ; 2
    asl                     ; 2
;---------------------------------------
    sta     HMOVE           ; 3         @01     two cycles too early, ok
    sta     NUSIZ1          ; 3         @04     (>= @00)
.continueM1_a:              ;   = 23            timing is constant!

; draw car #1:
    lda     #CARHEIGHT-1    ; 2
    dcp     P1_Y            ; 5
    bcs     .doDraw1_a      ; 2
    lda     #0              ; 2
    BIT_W                   ;-1
.doDraw1_a:
    lda     (P1_Ptr),y      ; 5
    sta     GRP1            ; 3 = 18    @22

; set grenade color and clear PF0:
    lda     grenadeClr      ; 3
    sta     COLUPF          ; 3         @28
    stx     PF0             ; 3 =  9    @31     clear PF0 (bits 0..3: don't care)

; prepare tombstone row:
    lda     MaskTbl-1,x     ; 4
    and     dataPF_L-1,x    ; 4
    sta     tmpPF_L         ; 3
    lda     dataPF_R-1,x    ; 4
    sta     tmpPF_R         ; 3 = 18

    SLEEP   4               ; 4 =  4

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @56

; draw zombie #0:
    cpy     M0_YTmp         ; 3
    bpl     .checkM0_a      ; 2
;.drawM0_a:
    lda     (M0_Ptr),y      ; 5
    sta     HMM0            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     HMOVE           ; 3         @75     four cycles too early, (just!) ok
;---------------------------------------
    sta     NUSIZ0          ; 3         @03     (>= @00)
.continueM0_a:              ;   = 23            timing is constant!

; draw car #0:
    lda     #CARHEIGHT-1    ; 2
    dcp     P0_Y            ; 5
    bcs     .doDraw0_a      ; 2
    lda     #0              ; 2
    BIT_W                   ;-1
.doDraw0_a:
    lda     (P0_Ptr),y      ; 5
    sta     GRP0            ; 3 = 18    @21

; set color for second ball row:
    lda     #$0e            ; 2
    sta     COLUPF          ; 3 =  5    @26     (<= @27)

; tombstone indexer routine (part 2/2):
    dex                     ; 2
    bmi     .exitLoop       ; 2=  4

; disable grenade:
    lda     grenadeClr      ; 3
    asl                     ; 2
    sta     ENABL           ; 3         @38     VDELed!
    lda     #$90            ; 2
    sta     PF0             ; 3 = 13    @43

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @46

; TombstoneIndexer routine:
    dey                     ; 2
    lda     #$0e            ; 2
    jmp     .nextBlock      ; 3 =  7

.exitLoop:
    jmp     ExitKernel
;***************************************

.disableM1_a:               ; 9
    SLEEP   2               ; 2                 waste two cycles for constant timing
    lda     #DISABLE_BM     ; 2
    beq     .continueM1a_a  ;10

.checkM1_a:                 ; 6
    bne     .disableM1_a    ; 2
;.enableM1_a:
    lda     M1_Y            ; 3
    sta     M1_YTmp         ; 3
    lda     #ENABLE_BM      ; 2
.continueM1a_a:
;---------------------------------------
    sta.w   ENAM1           ; 4         @01     (>= @00)
    bpl     .continueM1_a   ; 3

.disableM0_a:               ; 8
    SLEEP   2               ; 5                 waste two cycles for constant timing
    lda     #DISABLE_BM     ; 2
    beq     .continueM0a_a  ; 9

.checkM0_a:                 ; 6
    bne     .disableM0_a    ; 2
;.enableM0_a:
    lda     M0_Y            ; 3
    sta     M0_YTmp         ; 3
    lda     #ENABLE_BM      ; 2
.continueM0a_a:
;---------------------------------------
    sta.w   ENAM0           ; 4         @02     (>= @00)
    bpl     .continueM0_a   ; 3
;***************************************


ExitKernel:
    lda     #0
    sta     WSYNC
;---------------------------------------
    sta     PF0             ;                   already 0
    sta     PF1
    sta     PF2
    sta     ENABL
    sta     GRP0
    sta     GRP1
    sta     GRP0
    sta     ENAM0
    sta     ENAM1

    ldx     #2
.waitScreen0:
    lda     INTIM
    bne     .waitScreen0
    sta     WSYNC
    stx     VBLANK
    rts


OverScan SUBROUTINE
    lda     #36
    sta     TIM64T

    lda     SWCHB
    lsr
    bcs     .skipReset
    brk
.skipReset:

    lda     #ENABLE_BM >> 1
    bit     SWCHB
    bmi     .fullScreen
    lda     #GRENADE_COL
.fullScreen:
    sta     grenadeClr

; some code for moving around:
    ldx     #1
    bit     INPT4
    bpl     .moveZombies

    lda     frameCnt
    and     #$03
    beq     .startMoveCars
    jmp     .skipJoystick

.startMoveCars:
    lda     SWCHA
.loopMoveCars:
    lsr
    ldy     P0_YFullres,x
    bcs     .skipUpCar
    bne     .skipResetUpCar
    ldy     #KERNELHEIGHT*2-1
.skipResetUpCar
    dey
.skipUpCar:

    lsr
    bcs     .skipDownCar
    iny
    cpy     #KERNELHEIGHT*2-1
    bne     .skipDownCar
    ldy     #0
.skipDownCar:
    sty     P0_YFullres,x

    lsr
    ldy     P0_X,x
    bcs     .skipLeftCar
    bne     .skipResetLeftCar
    ldy     #SCREEN_W
.skipResetLeftCar:
    dey
.skipLeftCar:

    lsr
    bcs     .skipRightCar
    iny
    cpy     #SCREEN_W
    bne     .skipRightCar
    ldy     #0
.skipRightCar:
    sty     P0_X,x

    dex
    bpl     .loopMoveCars
.   jmp     .skipJoystick

.moveZombies:
    lda     frameCnt
    and     #$03
    bne     .skipJoystick

    lda     SWCHA
.loopMoveZombies:
    ldy     M0_Y,x
    lsr
    bcs     .skipUpZombie
    iny
    cpy     #KERNELHEIGHT+1|$80
    bne     .skipUpZombie
    ldy     #1|$80
.skipUpZombie:

    lsr
    bcs     .skipDownZombie
    dey
    cpy     #0|$80
    bne     .skipDownZombie
    ldy     #KERNELHEIGHT|$80
.skipDownZombie:
    sty     M0_Y,x

    lsr
    ldy     M0_X,x
    bcs     .skipLeftZombie
    bne     .skipResetLeftZombie
    ldy     #SCREEN_W
.skipResetLeftZombie:
    dey
.skipLeftZombie:

    lsr
    bcs     .skipRightZombie
    iny
    cpy     #SCREEN_W
    bne     .skipRightZombie
    ldy     #0
.skipRightZombie:
    sty     M0_X,x

    dex
    bpl     .loopMoveZombies
.skipJoystick:

.waitTim:
    lda     INTIM
    bne     .waitTim
    rts


SetPosX SUBROUTINE
; TODO: use new Fatal Run method discovered by Manuel
; calculates the values and positions objects:
; Input:
; - a = x-position
    tay                      ; 2
    iny                      ; 2
    tya                      ; 2
    and    #$0F              ; 2
    sta    tmpVar            ; 3
    tya                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    tay                      ; 2
    clc                      ; 2
    adc    tmpVar            ; 3
    cmp    #$0F              ; 2
    bcc    .skipIny          ; 2
    sbc    #$0F              ; 2
    iny                      ; 2
.skipIny:
    sta    WSYNC             ; 3
    eor    #$07              ; 2
    asl                      ; 2
    asl                      ; 2
    asl                      ; 2
    asl                      ; 2
    sta.wx  HMP0,x           ; 5        @15
.waitPos:
    dey                      ; 2
    bpl    .waitPos          ; 2
.bankCheck:
    sta    RESP0,X           ; 4
    rts                      ; 6
;SetPosX END


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
    org     $f700

PF1Data  ;D    76543210
    .byte %01111001
    .byte %01000101
    .byte %01000101
    .byte %01000101
    .byte %01111001
    .byte %00000000
    .byte %01111001
    .byte %01000101
    .byte %01000101
    .byte %01000101
    .byte %01111001

PF2Data  ;EA   01234567
    .byte %00101111
    .byte %10100000
    .byte %11100111
    .byte %00100000
    .byte %11101111
    .byte %00000000
    .byte %00101111
    .byte %00100000
    .byte %11100111
    .byte %00100000
    .byte %11101111

PF2bData ;     76543210;<--- scanning order
    .byte %11011110
    .byte %00010001
    .byte %10011110
    .byte %01010001
    .byte %10011110
    .byte %00000000
    .byte %01000100
    .byte %01000100
    .byte %11000100
    .byte %01000100
    .byte %11011111

PF1bData;      01234567;<--- scanning order
    .byte %10111110
    .byte %00100000
    .byte %10111110
    .byte %10100010
    .byte %10100010
    .byte %00000000
    .byte %00100010
    .byte %00100010
    .byte %00111110
    .byte %00100010
    .byte %00100010

CarTbl:
     .byte  <CarRotation0 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT
     .byte  <CarRotation1 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT
     .byte  <CarRotation2 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT
     .byte  <CarRotation3 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT
     .byte  <CarRotation4 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT
     .byte  <CarRotation5 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT
     .byte  <CarRotation6 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT
     .byte  <CarRotation7 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT
     .byte  <CarRotation8 - TOMBSTONEROWS*TOMBSTONEHEIGHT - CARHEIGHT

Ped1_Tbl:
    .byte   <Ped1_1 + ZOMBIE_H-1
    .byte   <Ped1_2 + ZOMBIE_H-1
    .byte   <Ped1_3 + ZOMBIE_H-1
    .byte   <Ped1_4 + ZOMBIE_H-1

Pot2Tbl:
V SET 1
  REPEAT    8
    .byte V
V SET V + V
  REPEND
;    .byte   $01, $02, $04, $08, $10, $20, $40, $80

MaskTbl:
V SET %01010101
  REPEAT    TOMBSTONEROWS
    .byte V
V SET V ^ $ff
  REPEND

    org     $f700 + TOMBSTONEROWS*TOMBSTONEHEIGHT + CARHEIGHT

CarRotation0;UP (ACTUAL)
  ;.byte %00000000
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01100110; .XX..XX.
  .byte %00011000; ...XX...
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.
  .byte %00011000; ...XX...

CarRotation1
  ;.byte %00000000
  .byte %00000110; ....XX..
  .byte %00011110; ..XXXX..
  .byte %01110100; XXX.X...
  .byte %00011010; XX.XX.X.
  .byte %00011010; ...XX.X.
  .byte %01011111; .X.XXXXX
  .byte %00111100; ..XXXX..
  .byte %00101100; ..X.XX..

CarRotation2
  ;.byte %00000000
  .byte %00011000; ...XX...
  .byte %01111000; .XXXX...
  .byte %01000010; .X....X.
  .byte %11011011; XX.XX.XX
  .byte %11011100; XX.XXX..
  .byte %00011110; ...XXXX.
  .byte %00110110; ..XX.XX.
  .byte %00010000; ...X....

CarRotation3
  ;.byte %00000000
  .byte %00110000; ..XX....
  .byte %00110010; ..XX.X..
  .byte %01100011; .XX...XX
  .byte %01011110; .X.XXXX.
  .byte %11111111; XXXXXXXX
  .byte %11000111; XX...XXX
  .byte %00011100; ...XXX..
  .byte %00000100; .....X..

CarRotation4;RIGHT|LEFT
  ;.byte %00000000
  .byte %00000000; ........
  .byte %11101110; XXX.XXX.
  .byte %01100100; .XX..X..
  .byte %11011111; XX.XXXXX
  .byte %11011111; XX.XXXXX
  .byte %01100100; .XX..X..
  .byte %11101110; XXX.XXX.
  .byte %00000000; ........

CarRotation5
  ;.byte %00000000
  .byte %00000100; .....X..
  .byte %00011100; ...XXX..
  .byte %11000111; XX...XXX
  .byte %11111111; XXXXXXXX
  .byte %01011110; .X.XXXX.
  .byte %01100011; .XX...XX
  .byte %00110010; ..XX.X..
  .byte %00110000; ..XX....

CarRotation6
  ;.byte %00000000
  .byte %00010000; ...X....
  .byte %00110110; ..XX.XX.
  .byte %00011110; ...XXXX.
  .byte %11011100; XX.XXX..
  .byte %11011011; XX.XX.XX
  .byte %01000010; .X....X.
  .byte %01111000; .XXXX...
  .byte %00011000; ...XX...

CarRotation7
  ;.byte %00000000
  .byte %00101100; ..X.XX..
  .byte %00111100; ..XXXX..
  .byte %01011111; .X.XXXXX
  .byte %00011010; ...XX.X.
  .byte %11011010; XX.XX.X.
  .byte %11101000; XXX.X...
  .byte %00111100; ..XXXX..
  .byte %00001100; ....XX..

CarRotation8;DOWN (ACTUAL)
  ;.byte %00000000
  .byte %00011000; ...XX...
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.
  .byte %00011000; ...XX...
  .byte %01100110; .XX..XX.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.

Ped1_1;   OOOOWW__
   .byte HMOVE_L1|BM_SIZE_2/4 ; .XX.....
   .byte HMOVE_L1|BM_SIZE_2/4 ; ..XX....
   .byte HMOVE_0 |BM_SIZE_4/4 ; ...XXXX.
   .byte HMOVE_R2|BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_L3|BM_SIZE_4/4 ; .XXXX...
   .byte HMOVE_R1|BM_SIZE_1/4 ; ....X...
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XX...

Ped1_2;   OOOOWW__
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %00000000 ; ........
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...

Ped1_3;   OOOOWW__
   .byte %11110100 ; .XX.....
   .byte %00000000 ; ..X.....
   .byte %11011100 ; ...XXXX.
   .byte %11100100 ; ..XX....
   .byte %11010100 ; ...XX...
   .byte %00000000 ; ........
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...

   .byte HMOVE_0
Ped1_4;   OOOOWW__
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XXXX.
   .byte HMOVE_0 |BM_SIZE_1/4 ; ...X....
   .byte HMOVE_0 |BM_SIZE_4/4 ; ...XXXX.
   .byte HMOVE_R3|BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_L2|BM_SIZE_4/4 ; XXXX....
   .byte HMOVE_R1|BM_SIZE_4/4 ; ..XXXX..
   .byte HMOVE_0 |BM_SIZE_2/4 ; .XX.....
   .byte HMOVE_0 |BM_SIZE_2/4 ; .XX.....

    org $f7fc
    .word   Start
    .word   Start
